import { Canvas, CanvasRenderingContext2D } from "canvas";
import { AssetKind, AssetHelpSize, AssetPart, IAssetPartMetadata } from "./part";
import { ColorRGBA, IColor } from "../color";
/**
 * Represents any kind of dimension.
 * It is actually used to describe a lot of things.
 */
export type Dimensions = {
    w: number;
    h: number;
};
export type Position = {
    x: number;
    y: number;
};
/**
 * The interface `IAssetMetadata` defines the metadata information about an asset.
 */
export interface IAssetMetadata {
    baseSize: Dimensions;
    divisor: Dimensions;
    kind: AssetKind;
    name?: string;
}
export declare const DEFAULT_METADATA: {
    baseSize: {
        w: number;
        h: number;
    };
    divisor: {
        w: number;
        h: number;
    };
    kind: AssetKind;
};
/**
 * The `export interface ISave` defines an interface that specifies two methods:
 *
 * `save` and `saveAs`. These methods are used to save an asset to a file.
 */
export interface ISave {
    save: () => this;
    saveAs: (path: string) => this;
}
/**
 * This interface represents the miniman asset that can be used
 * in the whole library.
 *
 * IAsset inherits this interface.
 *
 * It has the basic method to manipulate an asset.
 */
export interface IMinimalAsset extends ISave {
    canvas: Canvas;
    readonly metadata: IAssetMetadata;
    setName: (name: string) => this;
    setColor: (color: IColor) => this;
    load: (path: string) => Promise<this>;
    loadFromCanvas: (canvas: Canvas) => this;
    empty: (assetHelpSize: AssetHelpSize) => this;
    scale: (assetHelpSize: AssetHelpSize) => this;
    restore: () => this;
}
export declare class MinimalAsset implements IMinimalAsset {
    canvas: Canvas;
    readonly metadata: IAssetMetadata;
    protected ctx: CanvasRenderingContext2D;
    protected originalCanvas: Canvas;
    constructor(metadata: IAssetMetadata);
    /**
     * This function returns the multiplier value based on the canvas width and base
     * size metadata.
     * @returns A scale factor
     */
    get multiplier(): number;
    /**
     * This function creates an empty canvas with a specified size and returns the
     * object.
     * @param {AssetHelpSize} assetHelpSize - `assetHelpSize`
     * @returns this
     */
    empty(assetHelpSize?: AssetHelpSize): this;
    /**
     * Scale the canvas using depending of `assetHelpSize`
     * @param assetHelpSize
     * @returns this
     */
    scale(assetHelpSize: AssetHelpSize): this;
    /**
     * This function loads a canvas and its context for further use.
     * @param {Canvas} canvas
     * @returns A Promise that resolves to the current instance of the class.
     */
    loadFromCanvas(canvas: Canvas): this;
    /**
     * This function loads an image from a given path and sets it as the canvas
     * background.
     * @param {string} path - A string representing the file path of an image file.
     * @returns Promise<this>
     */
    load(path: string): Promise<this>;
    /**
     * The function sets the name property of an object and returns the object
     * itself.
     * @param {string} name - The `name` parameter is a string that represents the
     * name to be set for a metadata object.
     * @returns this
     */
    setName(name: string): this;
    /**
     * The function restores the canvas and its context to their original state.
     * @returns The `restore()` method is returning the current instance of the
     * object (`this`) after restoring the canvas and its context to their original
     * state.
     */
    restore(): this;
    /**
     * This function saves an image file with the name of the metadata in PNG format.
     * @returns this
     */
    save(cropped?: boolean): this;
    /**
     * This function saves the canvas as an image file at the specified path.
     * @param {string} path - A string representing the file path where the canvas
     * should be saved. This can include the file name and extension.
     * @returns this
     */
    saveAs(path: string, cropped?: boolean): this;
    /**
     * TODO
     */
    setColor(_color: IColor): this;
}
export interface IAsset<T extends AssetPart = AssetPart> extends IMinimalAsset {
    multiplier: number;
    setVerification: (value: boolean) => this;
    getPartMetadata(assetPart: T): IAssetPartMetadata;
    getPartCanvas(assetPart: T): Canvas;
    colorPart: (color: IColor, assetPart: T) => this;
    colorParts: (color: IColor, ...assetParts: T[]) => this;
    copyPart: (asset: IAsset<T>, assetPart: T) => this;
    copyParts: (asset: IAsset<T>, ...assetParts: T[]) => this;
    setPartSaveDirectory: (directory: string) => this;
    savePart: (assetPart: T) => this;
    saveParts: (...assetParts: T[]) => this;
    saveAllParts(): this;
}
export declare abstract class Asset<T extends AssetPart> extends MinimalAsset implements IAsset<T> {
    canvas: Canvas;
    readonly metadata: IAssetMetadata;
    protected ctx: CanvasRenderingContext2D;
    protected originalCanvas: Canvas;
    private partSaveDirectory;
    private verification;
    private id;
    constructor(metadata: IAssetMetadata);
    /**
     * The function sets the verification value and returns the object instance.
     * @param {boolean} value - a boolean value that represents whether the
     * verification is set or not.
     * @returns this
     */
    setVerification(value: boolean): this;
    /**
     * This function returns the metadata of a given asset part.
     * @param {T} assetPart - Asset part
     * @returns Asset part metadata
     */
    protected _getPartMetadata(assetPart: T): IAssetPartMetadata;
    /**
     * This function returns scaled metadata for a given asset part.
     * @param {T} assetPart - Asset part
     * @returns Asset part metadata
     */
    getPartMetadata(assetPart: T): IAssetPartMetadata;
    /**
     *
     * @param assetPart Asset part
     * @returns Part as a Canvas
     */
    getPartCanvas(assetPart: T): Canvas;
    /**
     * This function checks if a canvas has a valid aspect ratio based on its width
     * and height.
     * @param {Canvas} [canvas]
     * @returns A boolean value indicating whether the canvas has a valid aspect
     * ratio or not.
     */
    private hasValidAspectRatio;
    /**
     * This function loads an image from a canvas and throws an error if the image
     * has the wrong aspect ratio.
     * @param {Canvas} canvas
     * @returns this
     */
    loadFromCanvas(canvas: Canvas): this;
    /**
     *
     * @param color Color
     * @param assetPart Asset part
     * @param callback A function applied on every canvas byte
     * @returns
     */
    protected _colorPart(color: IColor, assetPart: T, callback: (rgba: ColorRGBA, color: ColorRGBA) => void): this;
    /**
     * This function takes a color and an asset part, retrieves the image data for
     * that part, applies the color to the image data, and then puts the modified
     * image data back into the canvas.
     * @param {IColor} color - Color
     * @param {T} assetPart - Asset part
     * @returns this
     */
    colorPart(color: IColor, assetPart: T): this;
    /**
     * This function adds color parts to an asset.
     * @param {IColor} color - Color
     * @param {T[]} assetParts - Asset parts
     * @returns this
     */
    colorParts(color: IColor, ...assetParts: T[]): this;
    /**
     * This function sets the color of an asset based on the provided color and
     * metadata.
     * @param {IColor} color - Color
     * @returns this
     */
    setColor(color: IColor): this;
    /**
     * This function copies a specific part of an asset onto another asset.
     * @param {IAsset} asset - The asset parameter is an object of type IAsset, which
     * represents an image.
     * @param {T} assetPart - Asset part
     * @returns this
     */
    copyPart(asset: IAsset<T>, assetPart: T): this;
    /**
     * The function copies parts of an asset to another asset.
     * @param {IAsset} asset - The asset parameter is an object of type IAsset, which
     * represents an image.
     * @param {T[]} assetParts - Asset parts
     * @returns this
     */
    copyParts(asset: IAsset<T>, ...assetParts: T[]): this;
    /**
     * This function sets the save directory for a part and returns the object
     * instance.
     * @param {string} directory - The `directory` parameter is a string that
     * represents the path to the directory where the saved parts will be stored.
     * @returns this
     */
    setPartSaveDirectory(directory: string): this;
    /**
     * This function generates a cache key for a given asset part based on the
     * metadata and ID.
     * @param {T} assetPart - Asset part
     * @returns A cache key
     */
    protected getPartCacheKey(assetPart: T): string;
    /**
     * This function saves a part of an asset as a PNG file.
     * @param {T} assetPart - Asset part
     * @returns this
     */
    savePart(assetPart: T): this;
    /**
     * The function saves multiple asset parts and returns the object it belongs to.
     * @param {T[]} assetParts - Asset parts
     * @returns this
     */
    saveParts(...assetParts: T[]): this;
    saveAllParts(): this;
}
