import { Canvas } from "canvas";
import { Asset, MinimalAsset } from "./base";
import { EmoticonPart, EyeSkinPart, SkinPart, WeaponGameSkinPart } from "./part";
import { IColor } from "../color";
import Gameskin from "./gameskin";
import Emoticon from "./emoticon";
export default class Skin extends Asset<SkinPart> {
    renderCanvas: Canvas;
    private renderCtx;
    private eyeAssetPart;
    private _orientation;
    constructor();
    /**
     * This function sets the eye asset part.
     * @param {EyeSkinPart} value - EyeSkinPart - a type of object representing a
     * part of an eye skin asset.
     * @returns this
     */
    setEyeAssetPart(value: EyeSkinPart): this;
    /**
     * Gets orientation
     */
    get orientation(): number;
    /**
     * Defines the direction in which the tee looks
     * @param value - Angle
     * @returns this
     */
    setOrientation(value: number): this;
    /**
     * Get the real eye position on the tee
     * @param x
     * @param y
     * @returns A position
     */
    private getEyePosition;
    /**
     * The function returns a cloned canvas object.
     */
    get cloneRenderCanvas(): Canvas;
    loadFromCanvas(canvas: Canvas): this;
    colorPart(color: IColor, assetPart: SkinPart): this;
    /**
     * Applies `color` on the tee body, hand and its eyes.
     * @param {IColor} color - Color
     * @returns this
     */
    colorBody(color: IColor): this;
    /**
     * The function sets the color of the body and feet of an object.
     * @param {IColor} bodyColor - Body color
     * @param {IColor} footColor - Foot color
     * @returns this
     */
    colorTee(bodyColor: IColor, footColor: IColor): this;
    /**
     * This function reorders the body of a skin image so that the average grey is
     * 192,192,192.
     *
     * https://github.com/ddnet/ddnet/blob/master/src/game/client/components/skins.cpp#L227-L263
     */
    private reorderBody;
    /**
     * This function draws a part of an asset onto a canvas using the provided
     * metadata.
     * @param {Canvas} canvas - Canvas
     * @param {IAssetPartMetadata} metadata - Asset part metadata
     * @returns this
     */
    private drawPart;
    /**
     * This function renders a tee.
     * @param {EyeSkinPart} [eyeAssetPart] - Skin part, must be a eye kind
     * @returns this
     */
    render(eyeAssetPart?: EyeSkinPart): this;
    /**
     * The function saves the current render as a PNG file with the name of the
     * metadata.
     * @returns this
     */
    saveRender(cropped?: boolean): this;
    /**
     * This function saves a canvas as an image file with a given path and returns
     * the object it was called on.
     * @param {string} path - A string representing the file path where the rendered
     * canvas should be saved.
     * @returns this
     */
    saveRenderAs(path: string, cropped?: boolean): this;
}
export declare class SkinFull extends MinimalAsset {
    private skin;
    private gameskin;
    private emoticon?;
    private weapon;
    private weaponMetadata;
    private emoticonPart;
    constructor();
    /**
     * Set a tee skin
     * @param value - A Skin object
     * @returns this
     */
    setSkin(value: Skin): this;
    /**
     * Set a weapon
     * @param value - WeaponGameSkinPart
     * @returns this
     */
    setWeapon(value: WeaponGameSkinPart): this;
    /**
     * Set a gameskin
     * @param value - Gameskin
     * @param part - WeaponGameSkinPart
     * @returns this
     */
    setGameskin(value: Gameskin, part?: WeaponGameSkinPart): this;
    /**
     * Set an emoticon
     * @param value - Emoticon, must be an emoticon
     * @param part - EmoticonPart optional
     * @returns this
     */
    setEmoticon(value: Emoticon, part?: EmoticonPart): this;
    /**
     * Set an emoticon part
     * @param value - Emoticon part
     * @returns this
     */
    setEmoticonPart(value: EmoticonPart): this;
    resetEmoticon(): this;
    /**
     * Put the tee hand on a weapon
     * @param weaponsCanvas - Weapon canvas
     * @returns this
     */
    private putHand;
    /**
     * Adjust the weapon position based on the tee body center.
     * @param origin - Origin position
     * @param orientation - Angle
     * @returns The new weapon position
     */
    private adjustWeaponPosition;
    /**
     * Put the weapon on the rendered tee canvas
     * @param orientation - Angle
     * @returns this
     */
    private putWeapon;
    private putEmoticon;
    /**
     * Creates a canvas with the rendered tee and its weapon.
     * @param orientation - Angle
     * @returns this
     */
    process(orientation?: number): this;
}
