import { Canvas, ImageData, CanvasRenderingContext2D } from 'canvas';
import Cache from '../cache';
import { Dimensions } from '../asset/base';
/**
 * The function draws a rounded rectangle on a canvas with a specified radius,
 * width, height, and color.
 * @param {CanvasRenderingContext2D} ctx - Canvas context
 * @param {number} r - The radius of the rounded corners of the rectangle.
 * @param {number} w - The width of the rounded rectangle.
 * @param {number} h - The height of the rounded rectangle.
 * @param {string} color - String color
 */
export declare function roundRect(ctx: CanvasRenderingContext2D, r: number, w: number, h: number, color: string): void;
/**
 * The function draws a rounded rectangle on a canvas context.
 * @param {CanvasRenderingContext2D} ctx - Canvas context
 * @param {number} x - The x-coordinate of the top-left corner of the rounded
 * rectangle.
 * @param {number} y - The `y` parameter in the `roundedImage` function represents
 * the vertical coordinate of the top-left corner of the rounded rectangle.
 * @param {number} width - The width of the rounded image.
 * @param {number} height - The height of the rounded rectangle that will be drawn
 * on the canvas.
 * @param {number} radius - The radius parameter is the radius of the corners of
 * the rectangle that will be drawn. It is used in the quadraticCurveTo() method to
 * create rounded corners.
 */
export declare function roundedImage(ctx: CanvasRenderingContext2D, x: number, y: number, width: number, height: number, radius: number): void;
/**
 * This function loads an image from a file path and creates a canvas with
 * the same dimensions as the image.
 * @param {string} path - A string representing the file path of the image
 * that needs to be loaded.
 * @returns Canvas Promise
 */
export declare function getCanvasFromFile(path: string): Promise<Canvas>;
/**
 * This function saves a canvas element as a PNG image to a specified path,
 * creating directories if they don't exist.
 * @param {string} path - A string representing the file path where the canvas will
 * be saved.
 * @param {Canvas} canvas - Canvas
 */
export declare function saveCanvas(path: string, canvas: Canvas): boolean;
/**
 * This function creates a canvas element from an ImageData object.
 * @param {ImageData} imageData - Image data
 * @returns A new canvas
 */
export declare function canvasFromImageData(imageData: ImageData): Canvas;
/**
 * Clones a given canvas element by creating a new canvas
 * with the same dimensions and drawing the original canvas onto it.
 * @param {Canvas} oldCanvas - A source canvas
 * @returns A new canvas
 */
export declare function cloneCanvas(oldCanvas: Canvas): Canvas;
/**
 * Flip a canvas
 * @param oldCanvas - A source canvas
 * @param flipX
 * @param flipY
 * @returns A new canvas
 */
export declare function canvasFlip(oldCanvas: Canvas, flipX?: boolean, flipY?: boolean): Canvas;
/**
 * Scale a cavas
 * @param oldCanvas - A source canvas
 * @param factor - Scale factor
 * @returns A new canvas
 */
export declare function scaleCanvas(oldCanvas: Canvas, factor: number): Canvas;
/**
 * Raws scale canvas
 * @param oldCanvas
 * @param factorW
 * @param factorH
 * @returns canvas
 */
export declare function rawScaleCanvas(oldCanvas: Canvas, factorW: number, factorH: number): Canvas;
/**
 *
 * @param oldCanvas - A source canvas
 * @param size - New canvas dimensions
 * @returns A new canvas
 */
export declare function resizeCanvas(oldCanvas: Canvas, size: Dimensions): Canvas;
/**
 * Rotate a canvas
 * @param oldCanvas - A source canvas
 * @param angle - Angle
 * @returns A new canvas
 */
export declare function rotateCanvas(oldCanvas: Canvas, angle: number, size?: number): Canvas;
/**
 *
 * @param canvas - A source canvas
 * @param from - Start side
 * @param to - End side
 * @returns A position on X or Y axis
 */
export declare function findOpaque(canvas: Canvas, from: number, to: number): number;
/**
 * Automatically find the smallest rectangle fitting with the image.
 * @param oldCanvas - A source canvas
 * @returns - A new canvas
 */
export declare function autoCropCanvas(oldCanvas: Canvas): Canvas;
/**
 * Global variable for the canvas cache;
 */
declare let cacheCanvas: Cache<Canvas>;
export { cacheCanvas };
